/*
	(C) 1997-2001 AROS - The Amiga Research OS
	$Id: addbuffers.c,v 1.1.1.1 2005/03/15 16:01:02 laire Exp $
 
	Desc: AddBuffers CLI command
	Lang: English
*/


/******************************************************************************
 
	NAME
 
		AddBuffers (drive) [(N)]
 
	SYNOPSIS
 
		DRIVE/A, BUFFERS/N
 
	LOCATION
 
		Workbench:C
 
	FUNCTION
 
		Add buffers to the list of available buffers for a specific
	drive. Adding buffers speeds disk access but has the drawback
	of using up system memory (512 bytes per buffer). Specifying
	a negative number subtracts buffers from the drive.
	    If only the DRIVE argument is specified, the number of 
	buffers for that drive are displayed without changing the buffer
	allocation.
 
	INPUTS
 
		DRIVE    --  the drive to alter the buffer allocation of
	BUFFERS  --  the number of buffers to add (or subtract in case of
	             a negative number) to a drive.
 
	RESULT
 
	NOTES
 
	EXAMPLE
 
	BUGS
 
	SEE ALSO
 
	INTERNALS
 
	HISTORY
 
******************************************************************************/

#include <stdio.h>
#include <exec/devices.h>
#include <exec/io.h>
#include <exec/memory.h>
#include <exec/semaphores.h>
#include <dos/dos.h>
#include <dos/exall.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include "addbuffers_version.h"

static const char version[] = VERSTAG;

#define ARG_TEMPLATE "DRIVE/A,BUFFERS/N"

enum
{
	ARG_DRIVE = 0,
	ARG_BUFFERS,
	NOOFARGS
};


int main(void)
{
	struct ExecBase *SysBase=*((struct ExecBase**) 4);
	struct DosLibrary	*DOSBase;
	IPTR           args[NOOFARGS] = { (IPTR)NULL, (IPTR)0 };
	struct RDArgs *rda;
	int    result;
	int    error = RETURN_OK;
	LONG   buffers = 0;

	if ((DOSBase=(struct DosLibrary *)OpenLibrary("dos.library",37))!=0)
	{
		rda=(struct RDArgs *)ReadArgs(ARG_TEMPLATE, args, NULL);

		if (rda != NULL)
		{
			STRPTR  drive = (STRPTR)args[ARG_DRIVE];
			LONG  *bufsptr = (LONG *)args[ARG_BUFFERS];

			if (bufsptr != NULL)
			{
				buffers = *bufsptr;
			}

			result = AddBuffers(drive, buffers);

			if (result == -1)
			{
				Printf("%s has %ld buffers\n", (ULONG)drive, IoErr());
			}
			else if (result > 0)
			{
				Printf("%s has %ld buffers\n", (ULONG)drive, (LONG)result);
			}
			else
			{
				PrintFault(IoErr(), "AddBuffers");
				error = RETURN_FAIL;
			}

			FreeArgs(rda);
		}
		else
		{
			PrintFault(IoErr(), "AddBuffers");
			error = RETURN_FAIL;
		}

		CloseLibrary((struct Library *)DOSBase);
	}
	return error;
}
